#encoding:UTF8
from BaseDataMaintenance.dataSource.pool import ConnectorPool
from BaseDataMaintenance.dataSource.source import *
from BaseDataMaintenance.common.Utils import *
import queue
from tablestore import *
from multiprocessing import RLock
from threading import Thread
from apscheduler.schedulers.blocking import BlockingScheduler
from BaseDataMaintenance.common.multiThread import MultiThreadHandler
from BaseDataMaintenance.model.ots.proposedBuilding_tmp import proposedBuilding_tmp
from BaseDataMaintenance.model.ots.designed_project import designed_project
import traceback

class DataSynchronization():

    def __init__(self):
        self.done_lock = RLock()
        self.isDone = False
        self.proposedBuilding_table = "proposedBuilding_tmp"
        self.proposedBuilding_table_index = "proposedBuilding_tmp_index"
        self.ots_client = getConnect_ots()

    def producer(self,task_queue):
        '''
        :return:生产数据
        '''
        ots_client = getConnect_ots()

        bool_query = BoolQuery(must_queries=[ExistsQuery("crtime")])

        columns = ["uuid","crtime","json_list_group"]

        rows, next_token, total_count, is_all_succeed = ots_client.search(self.proposedBuilding_table, self.proposedBuilding_table_index,
                                                                          SearchQuery(bool_query ,sort=Sort(sorters=[FieldSort("crtime",SortOrder.DESC)]), limit=100, get_total_count=True),
                                                                          ColumnsToGet(columns,return_type=ColumnReturnType.SPECIFIED))
        list_data = getRow_ots(rows)
        for _data in list_data:
            _proposed = proposedBuilding_tmp(_data)
            task_queue.put(_proposed,True)
        _count = len(list_data)
        while next_token:
            rows, next_token, total_count, is_all_succeed = ots_client.search(self.proposedBuilding_table, self.proposedBuilding_table_index,
                                                                              SearchQuery(bool_query ,next_token=next_token, limit=100, get_total_count=True),
                                                                              ColumnsToGet(columns,return_type=ColumnReturnType.SPECIFIED))
            list_data = getRow_ots(rows)
            for _data in list_data:
                _proposed = proposedBuilding_tmp(_data)
                task_queue.put(_proposed,True)
            _count += len(list_data)
            if _count>3000:
                break


    def comsumer(self,task_queue):

        def _handle(_proposed,result_queue,ots_client):

            print(_proposed)

            #修改designed_project
            _time = time.time()
            _project_dict = _proposed.toDesigned_project(ots_client)
            log("toDesigned_project takes %.2fs"%(time.time()-_time))

            try:
                _time = time.time()
                if _project_dict is not None:
                    #更新数据
                    _designed_project = designed_project(_project_dict)
                    _designed_project.update_project(ots_client)

                #删除tmp
                _proposed.delete_row(ots_client)
                log("update designed takes %.2fs"%(time.time()-_time))
            except Exception as e:
                log("comsumer failed cause of %s"%(str(e)))
                log(traceback.format_exc())


        result_queue = queue.Queue()

        mt = MultiThreadHandler(task_queue,_handle,result_queue,thread_count=30,ots_client=self.ots_client)
        mt.run()


    def waitTask(self,task_queue):
        for i in range(60):
            if task_queue.qsize()>0:
                return True
            else:
                time.sleep(1)
        return False

    def maxcompute2ots(self):

        task_queue = queue.Queue()

        self.producer(task_queue)

        # _dict = {"uuid":"12313","crtime":123,
        #                 "json_list_group":'''
        #                 [{"docid": 254681903, "shenpi_id": null, "type_id": "254681903", "page_time": "2022-07-25", "province": "云南", "city": null, "district": null, "tenderee": "景谷傣族彝族自治县永平镇人民政府", "tenderee_contact": "", "tenderee_phone": "0879-5311786", "agency": "云南赛林工程管理咨询有限公司", "project_code": "云赛招字2022-245", "project_name": "永平镇芒腊村芒腊组组内道路建设及基础设施建设扶持项目", "doctitle": "云赛招字2022-245：景谷傣族彝族自治县永平镇人民政府永平镇芒腊村芒腊组组内道路建设及基础设施建设扶持项目施工竞争性磋商成交公告", "docchannel": "101", "stage": "施工在建", "proportion": "全长1959.8m", "projectDigest": "标的信息，工程类：标段名称：永平镇芒腊村芒腊组组内道路建设及基础设施建设扶持项目。工程类：名称：永平镇芒腊村芒腊组组内道路建设及基础设施建设扶持项目。工程类：施工工期：90日历天", "projectAddress": null, "begin_time": null, "end_time": null, "project_name_refind": "永平镇芒腊村芒腊组组内道路建设及基础设施建设扶持项目", "industry": "司法建筑", "new_enough": 1, "follow_enough": 1}]
        #                 '''}
        # task_queue.put(proposedBuilding_tmp(_dict))

        self.comsumer(task_queue)


    def scheduler(self):
        _scheduler = BlockingScheduler()
        _scheduler.add_job(self.maxcompute2ots,"cron",minute="*/1")
        _scheduler.start()

def startSychro():
    ds = DataSynchronization()
    ds.scheduler()

if __name__=="__main__":
    ds = DataSynchronization()
    # ds.scheduler()
    ds.maxcompute2ots()



